;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x9f01d60d376bd7f33095c77d03de54b6 . "./code.cg2js/cg-common-control.fasl")
(#x72aa2276528deebec656d4731c3be30d . "./code.cg2js/cg-header-control.fasl")
(#x88408021da63ee9709f372f607b3383c . "./code.cg2js/cg-clipboard-stack.fasl")
(#x2c871ce4d91d5f546ea1b71cc1154a39 . "./code.cg2js/cg-yes-no-string.fasl")
(#x7199ffe82c250e718a209970f462929a . "./code.cg2js/cg-clipboard.fasl")
(#xa11aced25e9b54d8b67b619d30fd7094 . "./code.cg2js/cg-dde-utils.fasl")
(#x9aacc70c4fd6f03146abd835b0ab2e7f . "./code.cg2js/cg-editable-text.fasl")
(#x5a6a621f72e879de1fe7cab44bb8cbb8 . "./code.cg2js/cg-text-or-combo.fasl")
(#x974bb8fd413a7be4fad437114442788c . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x3e7bc6cb3c1d6e449cc2446932eee8c2 . "./code.cg2js/cg-graphics-context.fasl")
(#xf5fa872719c4bda5c9ea03522b7856cd . "./code.cg2js/cg-choose-printer.fasl")
(#x3158a219641a5425e44613aa64fd5128 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x0dd04ed3bbd120f764359bf3d140250f . "./code.cg2js/cg-edit-in-place.fasl")
(#xf5ca5a42e385d90991ab3c35ce67799d . "./code.cg2js/cg-split-bar.fasl")
(#xd2ac637224709be5d0509fa46246a1d7 . "./code.cg2js/cg-timer.fasl")
(#xf292098ebe272fbd4509c47c8106dce0 . "./code.cg2js/cg-progress-indicator.fasl")
(#xaf005214d8a847e0c242ae4f6cbef4c1 . "./code.cg2js/cg-object-editor.fasl")
(#x22288bd42d99a99a2a340704db1fcf20 . "./code.cg2js/cg-fill-texture.fasl")
(#xd77087d143d436035cd06284c53a21b5 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x43295fa8a22b8472c7f58e38c6aba018 . "./code.cg2js/cg-os-window.fasl")
(#xdb9b99d835f587b9d725cbd0d9c8af8e . "./code.cg2js/cg-color-dialog.fasl")
(#x6e0bba854c15d2c267bb4eda8d184255 . "./code.cg2js/cg-combo-box.fasl")
(#xeed480d390829d2961bf8e155bcb5a0f . "./code.cg2js/cg-directory-dialog.fasl")
(#xd4f0f601f19d4445327923c5be0e8fc1 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x255689f56342667c07175ec73f153881 . "./code.cg2js/cg-tooltip.fasl")
(#x5acc31e4d0773d3ce78797540dff4e07 . "./code.cg2js/cg-lettered-menu.fasl")
(#xffb722835c8ddb57b5ecdbd29a0ad6c9 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x89581cbd16b013029483d73b3105e84d . "./code.cg2js/cg-pixmap.fasl")
(#x953b79c090bd9a861b34ea49a01d6b68 . "./code.cg2js/cg.fasl")
(#x92f7b343d952d3cbf5d6a542cebfc22a . "./code.cg2js/cg-class-slot-grid.fasl")
(#x3e46a2a84499941140c76aed2bc9c190 . "./code.cg2js/cg-template-string.fasl")
(#xc4cba8b09fbd16ba8ff5d6c836b27145 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xcc2f853fc1aabbfa10405dfc88bd2073 . "./code.cg2js/cg-check-box.fasl")
(#xf6877ebfd1d36b90686281cd5f4a7ede . "./code.cg2js/cg-toggling-widget.fasl")
(#xbfe5f5a8cba41e778b370246e34a60bb . "./code.cg2js/cg-radio-button.fasl")
(#xdbe225df047a541c0d36abaa65722d0f . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xb4359982ca337c254a7285b145527073 . "./code.cg2js/cg-scaling-stream.fasl")
(#x3948dd8bc7692786a67d0a99b61b0d9b . "./code.cg2js/cg-multi-picture-button.fasl")
(#x0e0cc9bef09b3379884def03be03ca4a . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xcd9a06e589a5dea43460eb34ff157cfb . "./code.cg2js/cg-grid-widget.fasl")
(#x3f87241363fa0df26d9171cde9dca05a . "./code.cg2js/cg-text-edit-pane.fasl")
(#x95270540303b783878420b8a98c4c940 . "./code.cg2js/cg-item-list.fasl")
(#x6d698e671ab59446ac6a5c7bb21060ed . "./code.cg2js/cg-paren-matching.fasl")
(#x28721725dbc1e3be8928b2b416a75728 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xa321b6e54ae3312f749b49ac3afa0a4e . "./code.cg2js/cg-html-dialog.fasl")
(#xbc27904e7db09fa7c847f2d342af7053 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x9c5fa9baef6eb5d86d561e75e0edb314 . "./code.cg2js/cg-project-window.fasl")
(#x8354a4241665e3e3fe7f8cfb3d8795a0 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x63f7ce35368fe5e0734b7339338d0870 . "./code.cg2js/cg-toolbar.fasl")
(#x4a42d26323768842224d26fd7fdc5337 . "./code.cg2js/cg-caret.fasl")
(#xb35751d7e62b13f05176232af11d98b4 . "./code.cg2js/cg-scroll-bar.fasl")
(#x149706c876d6ac5158a82f7bce8bfc6b . "./code.cg2js/cg-web-browser.fasl")
(#x4becf363aac2897e43726efd92b3c9af . "./code.cg2js/cg-dropping-outline.fasl")
(#x8f512d593458a4f965092e686f27a58c . "./code.cg2js/cg-font-dialog.fasl")
(#x60b05ca113dab173f00558dbf4bbf91b . "./code.cg2js/cg-icon-pixmap.fasl")
(#x77ebe6c137495d7d6e3b31b7284dc683 . "./code.cg2js/cg-yes-no-list.fasl")
(#x819ca8d07860e3283b9b7cde9673033c . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x7a284b37231ec3d60b162d9b5a1fd8a6 . "./code.cg2js/cg-lamp.fasl")
(#xe2cc2215393f7722a2eaf27b02e230ef . "./code.cg2js/cg-drawable.fasl")
(#x740624bcef6e999a276cb28fc5ebf52a . "./code.cg2js/cg-choice-list.fasl")
(#x1674704b3b02a4945afcf1ee498de150 . "./code.cg2js/cg-curve.fasl")
(#xeceeb52ed7ddff21ba59cb93c05ccc50 . "./code.cg2js/cg-base.fasl")
(#xc1eb4455002ba7d589a638ca0c680967 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x69f7462a705d49662badc56d420bce01 . "./code.cg2js/cg-printing.fasl")
(#x317997a8d4058a6443d26d5674487dc5 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x31c793da5e62f5b01fb484237ec42d7b . "./code.cg2js/cg-hotspot.fasl")
(#x3df453c0d6941bab29408efc201a436e . "./code.cg2js/cg-object-editor-layout.fasl")
(#xd23275406100c5b99196bd78c494d5ff . "./code.cg2js/cg-utility-dialog.fasl")
(#x553225a5d6490bfb5ab5620a6a326ed3 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x071e7fc2683fddeeb80bcf17dff8a7ea . "./code.cg2js/cg-comtab.fasl")
(#x6de952bb8a54c8a8731871a734c1837b . "./code.cg2js/cg-os-widget.fasl")
(#x409f87e990a096b7eba38e368b674b53 . "./code.cg2js/cg-icon.fasl")
(#xa55a8e4480670ac781ef57d20df6480e . "./code.cg2js/cg-dev.fasl")
(#x774decaba83f2b4c5a751b021de779c5 . "./code.cg2js/cg-calendar.fasl")
(#xc022fad088284f10e740712082701ab0 . "./code.cg2js/cg-group-box.fasl")
(#xbf3c0f295f5f30ca432a0596717188ce . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xeca71c0993a85208a6f973f998533b46 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xa38da4d7073f9cd568da56fe38ff8fb2 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xc5858ca902b33e60d2f9af175dbe30ab . "./code.cg2js/cg-nodes.fasl")
(#xd7ef4d849869fc0955f30de90b15b054 . "./code.cg2js/cg-intersect.fasl")
(#x8c578827766c153142b496b5866a1cbc . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xf8005b50c3307a4b36c8f6acc8ef8a97 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x22e1368ff356a503dea43aea7e4c6007 . "./code.cg2js/cg-class-support.fasl")
(#x135d0f672a2fcf97a854184080861917 . "./code.cg2js/cg-dialog-item.fasl")
(#x8d40539b3682aa03a56a03d46558374a . "./code.cg2js/cg-plot-widget.fasl")
(#x9bfdea8407b344db03feae9468a554c1 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xdb4299cd74009e6304b20453b58759fb . "./code.cg2js/cg-up-down-control.fasl")
(#x2a4ac13c14a45072475ee664e3a95202 . "./code.cg2js/cg-rich-edit.fasl")
(#x8c038a9dc53413826a68d875b42ca93a . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x8c14c43b100fcc42fb1ee60ea89c0862 . "./code.cg2js/cg-status-bar.fasl")
(#x72f3a9f40f26f1bb02ba582d25be4bf2 . "./code.cg2js/cg-class-grid.fasl")
(#xf1b94e6be98ee8bed1c9c84af4dccdc0 . "./code.cg2js/cg-wrap-string.fasl")
(#xc949ef0a834fca27272b521209e89662 . "./code.cg2js/cg-layout.fasl")
(#xc659497cf413baa297254c6c6df93306 . "./code.cg2js/cg-lisp-widget.fasl")
(#x81b6952d5de8b62797ff9d7b92569e0c . "./code.cg2js/cg-button.fasl")
(#x38ee13f75f090bdecb858ba6e97395d1 . "./code.cg2js/cg-string-dialog.fasl")
(#x9abf93188bae103d81f341e713dd2301 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x7f6fe1cb04982c2dfaa94d990016f9fb . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x6ad53e5889b7fa6e4cc532f13409d851 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x07940ffa1a30be30c261715121ff3686 . "./code.cg2js/cg-static-text.fasl")
(#xb0c417cf4329854a9f6a22abe9036ddf . "./code.cg2js/cg-outline.fasl")
(#x3c9b4be83a841023144a80f2de9d7d7f . "./code.cg2js/cg-gesture-emulation.fasl")
(#x3b28ed7b2e512349810b73f283851451 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x080bc377c7d1dba39b82ef955ce624f8 . "./code.cg2js/cg-picture-widget.fasl")
(#xe42045282db69d4ec22e29003ba03a98 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x95c5dc82dd2498dc9848442a2fc73e92 . "./code.cg2js/cg-chart-widget.fasl")
(#x5827920a4c5e3fa8b3b666980dcddb3a . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xbb15901babbcdcc3a1149c9b48c1239f . "./code.cg2js/cg-lisp-text.fasl")
(#x04421e9b4fcce4bdb52aac8cbe5b397a . "./code.cg2js/cg-palette.fasl")
(#x44e320a8d994627646d52eb02b1cf6df . "./code.cg2js/cg-property.fasl")
(#x5a8050b4350d54a533371dd327b4f796 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x2a06c50fd53729d57b5c31a3aeeccc96 . "./code.cg2js/cg-file-dialog.fasl")
(#x303a692d21a256a6563fa89a696da6a9 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x191c51f0fd570d9820c79eb5ec57e1e1 . "./code.cg2js/cg-get-pixmap.fasl")
(#xd34cf26a898b53c32f9663922e90948e . "./code.cg2js/cg-tray.fasl")
(#xc663e5486f9e4eb6dc4eb714f9b7aaae . "./code.cg2js/cg-acache.fasl")
(#x214977862a842b5ccf18120010ee09a3 . "./code.cg2js/cg-message-dialog.fasl")
(#xc26458c36d1979336ead6debf5b37eb9 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xc0bd1047cd7d7663bf7cd9a416036d83 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x130350fb911d658037c0f96a6eb6601b . "./code.cg2js/cg-tab-control.fasl")
(#xcb792582822a4c8ade9277edd45196a1 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xe2bebae8abf1b283830fc8cbf90aafa2 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x0c88b8a20e53697b9700fdac7ed0dd74 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xfe13573a293469e4036096ac13c27a7c . "./code.cg2js/cg-menu.fasl")
(#xf014deafb0c9d299b836a90d310433f9 . "./code.cg2js/cg-get-position.fasl")
(#x7cca4771165c630dfeb48ff3c559ff94 . "./code.cg2js/cg-trackbar.fasl")
(#x680a4a724eb485d509a1deece1a71371 . "./code.cg2js/cg-selected-object.fasl")
(#xddd0f62d40d7130eaae3b5af7e37322b . "./code.cg2js/cg-list-view.fasl")
(#x079b25aa2577f0a0c8a2350ae9a53fe9 . "./code.cg2js/cg-html-widget.fasl")
(#x260cfb21087851c6a77d593fe913b5fe . "./code.cg2js/cg-sample-file-menu.fasl")
(#x2e727edfbbe99ee2f4599cb67252388a . "./code.cg2js/cg-text-widget.fasl")
(#xc3545868c236b4a81442ad174c3e6ed3 . "./code.idejs/ide-base.fasl")
(#x0d6632d7a93aa81af963b31daa8089bb . "./code.idejs/ide-process-browser.fasl")
(#x8f71948f708dc828c9ff83f0ba93cf56 . "./code.idejs/ide-patch-dialog.fasl")
(#x59b172be106703e0d06b4271ca43c98f . "./code.idejs/ide-keyshort.fasl")
(#x7ed854ee3606b38846eb794aa2102b95 . "./code.idejs/ide-window-list.fasl")
(#xfc0eb73b71094d30a7184b44891e8ff5 . "./code.idejs/ide.fasl")
(#x2f0737088a4212702a7475f281d02755 . "./code.idejs/ide-editor.fasl")
(#xef67397d04596b196055f2f956f69dd7 . "./code.idejs/ide-clipboard.fasl")
(#x337be49100a7d4270bb53eca8ea5f277 . "./code.idejs/ide-trace.fasl")
(#xc3b87f607664e76883e1e2b7e35ef14b . "./code.idejs/ide-class-browser.fasl")
(#x99a4d009d23ab8add24c4e1dffaac1ae . "./code.idejs/ide-grapher.fasl")
(#x9bbb454db2213a4aa93f5ba41535cbc2 . "./code.idejs/ide-apropos.fasl")
(#xf7db8be1cd48896eb9d0412e1c53db14 . "./code.idejs/ide-navigator.fasl")
(#xb7be7d72b8e9d1e06fa01a374a5855ea . "./code.idejs/ide-find-definitions.fasl")
(#xc937baf534883125e1f4ef2d5f5eafc8 . "./code.idejs/ide-project.fasl")
(#x881a1e2cbb1a1e050e656279bd7449f1 . "./code.idejs/ide-class-grapher.fasl")
(#x0bddb9443a112da90bb126a7d0362ca9 . "./code.idejs/ide-acache.fasl")
(#x320112700137ea047706579fedc63192 . "./code.idejs/ide-find-in-files.fasl")
(#xdfbf59e17cc303b28dfa2471e95a4103 . "./code.idejs/ide-inspector.fasl")
(#x7bdb0b838943781c5329ebbf3334f7e6 . "./code.idejs/ide-intro.fasl")
(#x187191b86f085dd4a0d51bbe6d78b3ed . "./code.idejs/ide-package-list.fasl")
(#xe3a8f1f394ef40f948993e46f14c2a57 . "./code.idejs/ide-dev.fasl")
(#xe76f94cd539a9422bcf339a54c01bf33 . "./code.idejs/ide-form.fasl")
(#xb19a66ec109be5a69fb5293c577fc104 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
